@@ -23,6 +23,7 @@ def main():
     credentials = parser.add_argument_group(title='credentials')
     credentials.add_argument('--email', type=str, help='webnovel email')
 
+    parser.add_argument('-v', '--verbose', help='enable animations', action='store_true')
     parser.add_argument('--threads', type=int, help='number of download threads', default=4)
     parser.add_argument('--timeout', type=int, help='webdriver timeout', default=60)
     parser.add_argument('--limit', type=int, help='amount of chapters to download')
@@ -66,6 +67,7 @@ def main():
 
     novelsave = NovelSave(args.action, directory=args.dir)
     novelsave.timeout = args.timeout
+    novelsave.verbose = args.verbose
 
     # get credentials
     if args.email is not None:
