@@ -1,7 +1,6 @@
 from queue import Queue
 
 from .action import ActionThread
-from .atomic import AtomicInt
 
 
 class ConcurrentActionsController:
@@ -20,14 +19,14 @@ class ConcurrentActionsController:
             for i in range(count)
         ]
 
-    def add(self, *args, block=False):
+    def add(self, *args, block=False, **kwargs):
         """
         put params to queue, equivalent to adding a task
 
         :param args: params for the task
         :param block: whether to do the operating while blocking
         """
-        self.queue_in.put(args, block=block)
+        self.queue_in.put((args, kwargs), block=block)
 
     def start(self):
         """
@@ -48,3 +47,15 @@ class ConcurrentActionsController:
             q_out = self.queue_out.unfinished_tasks == 0
 
         return q_in and q_out
+
+    def iter(self):
+        """
+        :return: generator that returns all the outputs
+        """
+        # start all the threads
+        self.start()
+
+        # wait for outputs from each thread
+        while not self.done:
+            yield self.queue_out.get()
+            self.queue_out.task_done()
