@@ -7,7 +7,11 @@ from yattag import Doc
 
 
 class Epub:
-    def create(self, novel, cover, volumes, chapters, save_path):
+    def create(self, novel, cover, volumes, chapters, save_path) -> Path:
+        # parameter validation
+        if not chapters:
+            raise ValueError("'chapters' may not be 'None'")
+
         # prepare data
         chapters.sort(key=lambda c: c.order)
 
@@ -61,7 +65,10 @@ class Epub:
 
         book.spine = [c for volume in book_chapters.values() for c in volume]
 
-        epub.write_epub(save_path / Path(f'{StringTools.slugify(novel.title)}.epub').resolve(), book, {})
+        path = save_path / Path(f'{StringTools.slugify(novel.title)}.epub').resolve()
+        epub.write_epub(path, book, {})
+
+        return path
 
     def _epub_chapter(self, chapter):
         """
